﻿unit U_ExtImages;

interface

{$i ..\extends.inc}
{$IFDEF FPC}
{$Mode Delphi}
{$ENDIF}

uses
{$IFDEF VERSIONS}
  fonctions_version,
{$ENDIF}
  Classes, U_ExtMapImageIndex,
  U_ExtImage, U_ExtCustomImages,
  u_framework_components,
  Controls,
  IniFiles,
{$IFDEF TNT}
     TntExtCtrls, TntStdCtrls,
{$ELSE}
     ExtCtrls, StdCtrls,
{$ENDIF}
     u_extcomponent;

const
  CST_DEFAULT_PANEL_WIDTH = 150;
  CST_DEFAULT_PANEL_HEIGHT = 100;
  CST_DEFAULT_PER_LINE = 4;
  CST_INI_DIRECTORY = 'Dir';
  CST_INI_EXT_OUT   = 'Out';
  CST_INI_MEMOS     = 'Memos';
{$IFDEF VERSIONS}
  gVer_TExtImages : T_Version = ( Component : 'Composant TExtImages' ;
                                  FileUnit : 'U_ExtImages' ;
                                  Owner : 'Matthieu Giroux' ;
                                  Comment : 'Présentation d''images de tous types avec résumé intégré dans le répertoire.' ;
                                  BugsStory : '1.0.0.0 : Scrolling and working. SubDirs not working.' + #13#10 +
                                              '0.9.0.0 : Creating from TExtImage.';
                                  UnitType : 3 ;
                                  Major : 1 ; Minor : 0 ; Release : 0 ; Build : 0 );

{$ENDIF}

type
  TExtPanelImage = class;
  TExtCollectionImages = class;

 { TExtMapImageIndex }

  { TExtPanelImage }

  TExtPanelImage = class(TExtCustomPanelImage)
  private
   s_Path : String;
   FPanelParent:TWinControl;
  protected
    function CreateMemo:{$IFDEF TNT}TTntCustomMemo{$ELSE}TCustomMemo{$ENDIF}; override;
    function CreateImage: TExtImage; override;
    procedure EraseMemo; override;
  public
   property Path : String read s_Path write s_Path;
  End;

  TExtPanelImageClass = class of TExtPanelImage;

  { TExtCollectionImages }

  TExtCollectionImages = class(TExtCustomCollectionImages)
  private
  public
  End;

{ TExtImages }

   TExtImages = class( TExtCustomImages )
     private
      FLastToUse : Boolean;
      FIniFileActive,
      FSubDirs ,
      FAutoLoad,
      FShowErrors : Boolean ;
      FErasedPos : Integer;
      FUnusedExtensions,
      FDirectories : TStringList;
      FCurrentFile,
      FDirectory : String;
      FIniFileMemos : TIniFile;
      FErasedImages : TStringList;
      procedure SetUnusedExtensions ( AValue : TStrings );
      function  GetUnusedExtensions : TStrings ;
     protected
       procedure AddTop; override;
       function CreateImagesMap : TExtCustomCollectionImages; override;
       procedure AddDirectory(const as_Directory: String; ab_SubDirs: Boolean);
       procedure p_SetDirectory ( const as_value : String );
       procedure AddFile (  const as_SourceFile : String );
       procedure DeleteImagesDown; override;
       procedure DeleteImagesUp; override;
       function  DeleteLinesDown:Boolean; override;
       procedure DeleteLinesUp; override;
       procedure AddImages; override;
       function  AddLines:Boolean; override;
      public
       constructor Create(AOwner: TComponent); override;
        destructor Destroy; override;
       procedure DestroyWnd; override;
       procedure Loaded; override;
       procedure DeleteIniMemos; virtual;
       procedure DeleteIni; virtual;
       procedure DeleteFromIni; virtual;
       procedure CreateSomeLines; override;
       procedure DestroyComponents; override;
       procedure ShowDirectory; virtual;
       property IniFileMemos : TIniFile read FIniFileMemos write FIniFileMemos;
       property Directories : TStringList read FDirectories;
       property ErasedImages : TStringList read FErasedImages;
       property CurrentFile : String read FCurrentFile;
     published
      property UnusedExtensions : TStrings read GetUnusedExtensions write SetUnusedExtensions;
      property IniFileActive : Boolean read FIniFileActive write FIniFileActive default True ;
      property AutoLoad : Boolean read FAutoLoad write FAutoLoad default True ;
      property SubDirs : Boolean read FSubDirs write FSubDirs default True ;
      property Directory : String read FDirectory write p_SetDirectory ;
      property ShowErrors : Boolean read FShowErrors write FShowErrors default True ;
     end;


implementation

uses
{$IFDEF FPC}
    unite_messages,
    LazFileUtils,
{$ELSE}
     unite_messages_delphi,
{$ENDIF}
     fonctions_init,
     Dialogs,
     fonctions_string,
     fonctions_system,
     sysutils,
     StrUtils,
     fonctions_file;

{ TExtPanelImage }

function TExtPanelImage.CreateMemo: TCustomMemo;
begin
  Result := TFWMemo.Create ((Collection as TExtCollectionImages).Component);
end;

function TExtPanelImage.CreateImage: TExtImage;
begin
  Result := TExtImage.Create ((Collection as TExtCollectionImages).Component);
end;

procedure TExtPanelImage.EraseMemo;
begin
  inherited;
  with (Collection as TExtCustomCollectionImages).Component as TExtImages do
   if FIniFileActive
   and assigned (FIniFileMemos) Then
     FIniFileMemos.EraseSection(copy(s_Path,1,posex('.',s_Path,length(s_path)-6)-1));
end;

{ TExtImages }

// initing SubDirs
constructor TExtImages.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FDirectories :=TStringList.Create;
  FErasedImages:=TStringList.Create;
  FUnusedExtensions:=TStringList.Create;
  FSubDirs:=True;
  FLastToUse := False;
  FIniFileActive := True;
  FIniFileMemos := nil;
  FShowErrors:=True;
  FAutoLoad:=True;
  FErasedPos := 0;
end;

destructor TExtImages.Destroy;
begin
  inherited Destroy;
  FIniFileMemos.Free;
  FDirectories .Destroy;
  FErasedImages.Destroy;
end;

procedure TExtImages.DestroyWnd;
var li_i : Integer;
begin
  if FIniFileActive Then
    Begin
      f_GetMemIniFile();
      if Assigned(FIniFileMemos) Then
        Begin
          // Erasing all before adding all
         for li_i := 0 to Columns.Count - 1 do
          with Columns [ li_i ] as TExtPanelImage do
            FIniFileMemos.EraseSection(copy(s_Path,1,posex('.',s_Path,length(s_path)-6)-1));

         // adding all without extension saving
         for li_i := 0 to Columns.Count - 1 do
          with Columns [ li_i ] as TExtPanelImage do
            if Trim(Memo.Lines.Text) > '' Then
              SauveTStringsDansIni ( FIniFileMemos, copy(s_Path,1,posex('.',s_Path,length(s_path)-6)-1), Memo.Lines );

        end;
      // fonction_init.FIniFile saving
      p_IniWriteSectionStr(Owner.Name,Name+CST_INI_DIRECTORY,FDirectory);
      SauveTStringsDansIni(FIniFile,Owner.Name+'.'+Name+CST_INI_EXT_OUT,FUnusedExtensions);
      // Saving
      fb_iniWriteFile( FIniFile );
      fb_iniWriteFile( FIniFileMemos );
    end;
  inherited;
end;

// Loads the directory
procedure TExtImages.Loaded;
var LSIniFile : String;
begin
  if FIniFileActive Then
   Begin
     f_GetMemIniFile();
     FIniFileMemos.Free;
     FIniFileMemos := TInifile.create ( fs_getIniDir () + fs_GetNameSoft () + CST_INI_MEMOS + CST_EXTENSION_INI);
   end;
  inherited Loaded;
  if FIniFileActive Then
    Begin
      FDirectory := f_IniReadSectionStr(Owner.Name,Name+CST_INI_DIRECTORY,FDirectory);
      LitTstringsDeIni(FIniFile,Owner.Name+'.'+Name+CST_INI_EXT_OUT,FUnusedExtensions);
    end;
  if not ( csDesigning in ComponentState)
  and ( FDirectory > '' )
  and FAutoLoad then
    ShowDirectory;
end;
procedure TExtImages.DeleteIniMemos;
var LSIniFile : String;
Begin
  LSIniFile := fs_getIniDir () + fs_GetNameSoft () + CST_INI_MEMOS + CST_EXTENSION_INI;
  if FileExistsUTF8(LSIniFile)
   Then DeleteFileUTF8(LSIniFile);
end;

procedure TExtImages.DeleteIni;
var LSIniFile : String;
Begin
  LSIniFile := f_getUserIniPath;
  if FileExistsUTF8(LSIniFile)
   Then DeleteFileUTF8(LSIniFile);
end;

procedure TExtImages.DeleteFromIni;
begin
  with f_GetMemIniFile do
    Begin
      EraseSection(Owner.Name+'.'+Name+CST_INI_EXT_OUT);
      EraseSection(Owner.Name);
    end;
end;

procedure TExtImages.AddDirectory(const as_Directory: String; ab_SubDirs : Boolean );
var lstl_Files : TStringList;
    li_i : Integer;
begin
  lstl_Files := TStringList.Create;
  try
      if fb_FindFiles ( lstl_Files, as_Directory, True, True, True, '*' ) Then
        Begin
         lstl_Files.Sort;
         with Columns do
          for li_i := 0 to lstl_Files.count-1 do
           try
             if (( ImagesPerLine = 0 ) and ( PanelLine.Left - Self.HorzScrollBar.Position > Self.Width ))
             or (( ImagesPerLine > 0 ) and ( PanelLine.Top  - Self.VertScrollBar.Position > Self.Height )  and (CounterPerLine = ImagesPerLine))
              Then
               Begin
                 FDirectories.Add(as_Directory);
                 FCurrentFile := lstl_Files  [ li_i ];
                 Break;
               end;
              if FSubDirs and DirectoryExistsUTF8 ( lstl_Files [ li_i ] ) Then
               AddDirectory ( lstl_Files [ li_i ], FSubDirs )
              Else
               AddFile(lstl_Files [ li_i ]);
           finally
           end;

      end;
  finally
    lstl_Files.Destroy;
  end;
end;

procedure TExtImages.ShowDirectory;
var lstl_Files : TStringList;
    li_i : Integer;
begin
  DestroyComponents;
  AddDirectory(IncludeTrailingPathDelimiter(FDirectory),FSubDirs);
end;

procedure TExtImages.DestroyComponents;
begin
  inherited;
  FLastToUse := False;
end;

procedure TExtImages.p_SetDirectory(const as_value: String);
begin
  if  ( FDirectory <> as_value ) Then
    Begin
      FDirectory:=as_value;
    end;
end;

// Add an image file to columns and panels
procedure TExtImages.AddFile(const as_SourceFile: String);
var li_iposext : Integer;
    ls_ext : String;
Begin
  if not fileexistsUTF8 ( as_SourceFile ) Then
   Exit;
  // unused extensions verify
  for li_iposext := 0 to FUnusedExtensions.Count - 1 do
   if posex ( FUnusedExtensions [ li_iposext ], as_SourceFile, Length(as_SourceFile)-6 ) > 0 Then
    Exit;
  // must keep li_iposext until except
  li_iposext := posex ( '.', as_SourceFile, Length(as_SourceFile)-6);
  if li_iposext = 0 Then
   Exit;
  ls_ext := copy ( as_SourceFile, li_iposext, Length(as_SourceFile) - li_iposext + 1 );
  ShowMessage(as_SourceFile);
  Begin
    // do we need to create a column ?
    if not FLastToUse Then
     Columns.Add;
    with Columns [ Columns.Count - 1 ] as TExtPanelImage do
      try
         // Adding or not adding : That is the question
        Image.ShowErrors:=False;
        Image.LoadFromFile ( as_SourceFile );
        If Image.Picture.Bitmap.Empty Then
          // adding for next
           FLastToUse := True
         Else
          // Adding really
           Begin
             FLastToUse := False;
             s_Path := as_SourceFile;
             if FIniFileActive
             and assigned(FIniFileMemos)
              Then
               LitTstringsDeIni ( FIniFileMemos, copy(as_SourceFile,1,posex(ls_ext,as_SourceFile,length(as_SourceFile)-6)-1), Memo.Lines );
           end;

      Except
        On E:Exception do
         Begin
          FLastToUse := True;
          if MessageDlg(fs_RemplaceMsg(gs_Images_Format_out,[ls_ext]),mtWarning,mbYesNo,0) = mrYes Then
            FUnusedExtensions.Add ( LowerCase(ls_ext));
         end;
      end;
  end;
end;

// procedure DeleteImagesDown
// Have got ErasedFiles to Delete when down
// When there is one line only
procedure TExtImages.DeleteImagesDown;
var Lparent: TWinControl;
begin
  if  ( Columns.Count > VisibleLines * ImagesPerLine )  Then
   Begin
    FErasedImages.Add((Columns [0] as TExtPanelImage).Path);
    Columns.Delete(0);
    Inherited; // Have Really deleted
   end;
end;

// procedure DeleteImagesUp
// Have got Directories and CurrentFile to Delete when up
// When there is one line only
procedure TExtImages.DeleteImagesUp;
begin
  if  ( Columns.Count > VisibleLines * ImagesPerLine )  Then
   Begin
    Columns.Delete(Columns.Count-1);
    Inherited; // Have Really deleted
   end;
end;

// procedure DeleteLinesDown
// Have got ErasedFiles to Delete when down
// Deleting one line of lines only
function TExtImages.DeleteLinesDown:Boolean;
var LParentTag,
    li_i : Integer;
    LParent : TWinControl;
begin
  with Columns do
   Begin
    // Testing if can delete lines
    Result :=( FErasedImages.Count = 0 ) or (Columns.CounterTop<Count);
    if Result
     Then
      /// deleting
      with Items [0] do
       Begin
         // Panel of the line
         LParent := Panel.Parent;
         LParentTag := Panel.Parent.Tag;
         //Deleting Panel of line
         for li_i := 0 to Count - 1 do
          with Items [0] as TExtPanelImage do
          Begin
            // Deleting one line only
            if LParentTag <> Panel.Parent.Tag
             Then Break;
            // Adding erased image to future added image
            FErasedImages.Add(Path);
            Delete(0);// deleting and destroying
            // Increasing and decreasing counters
            CounterTop:=CounterTop+1;
            Counter:=Counter-1;
          end;
         // Destroying panel of line
         LParent.Destroy;
         Inherited; // Have Really deleted
       end;
   end;
end;

// procedure DeleteLinesUp
// Have got Directories and CurrentFile to Delete when up
// Deleting one line of lines only
procedure TExtImages.DeleteLinesUp;
var LParentTag,
    li_i : Integer;
    LParent : TWinControl;
begin
  with Columns do
  // Testing if should delete lines
   if Count > VisibleLines * ImagesPerLine Then
    with Items [Count - 1] do
     Begin
       // Panel of the line
       LParent := Panel.Parent;
       LParentTag := Panel.Parent.Tag;
       /// deleting
       while Count > 1 do
        with Items [Count - 1] as TExtPanelImage do
        Begin
          // Deleting one line only
          if LParentTag <> Panel.Parent.Tag
           Then
             Break;
          // The next file will be the erased image
          FCurrentFile := s_Path;
          if ExtractFileDir(FCurrentFile)<>ExtractFileDir((Items[Count - 2] as TExtPanelImage).s_Path)
           Then // Not working fully
            FDirectories.Delete(FDirectories.IndexOf(ExtractFileDir(FCurrentFile)));
          Delete(Count - 1);
          // Decreasing counter
          Counter:=Counter-1;
        end;
       // Will add a line next
       CounterPerLine:=ImagesPerLine;
       // Destroying panel of line
       LParent.Destroy;
       Inherited; // Have Really deleted
     end;
end;

procedure TExtImages.AddTop;
var li_i : Integer;
begin
  with Columns do
   Begin
    for li_i := CounterTop-1 downto CounterTop-ImagesPerLine do
     Begin
      AddFile(FErasedImages[li_i]);
      if li_i = 0 then
        Break;
     end;
    CounterTop:=CounterTop-ImagesPerLine;
    PanelLine:=Items[Count-1].Panel.Parent as TPanel;
   end;
End;

procedure TExtImages.SetUnusedExtensions(AValue: TStrings);
begin
  FUnusedExtensions.Assign(AValue);
end;

function TExtImages.GetUnusedExtensions: TStrings;
begin
  Result:=FUnusedExtensions;
end;

procedure TExtImages.AddImages;
begin
  if  ( FErasedImages.Count > 0 )
  and ( Columns.CounterTop>0)
  Then
    Begin
      FLastToUse := False;
      AddTop;
    end;
  Inherited;
end;

function TExtImages.AddLines:Boolean;
begin
  with Columns do
  if  ( FErasedImages.Count > 0 )
  and (CounterTop>0)
  Then
    Begin
     FLastToUse := False;
     CreatePanelLine;
     PanelLine.Top:=0;
     CounterPerLine := 0;
     AddTop;
     Result := True;
    end
   Else
    Result := False;
  Inherited;
end;

procedure TExtImages.CreateSomeLines;
var lstl_Files : TStringList;
    li_i, li_j : Integer;
    LCurrentDir : String;
    lb_break : Boolean;
begin
  lb_break := False;
  lstl_Files := TStringList.Create;
  try
    while not lb_break and (FDirectories.Count > 0) and fb_FindFiles ( lstl_Files, FDirectories [ 0 ], True, True, True, '*' ) do
     Begin
       lstl_Files.Sort;
       with Columns do
        Begin
          LCurrentDir := FDirectories [ 0 ];
          FDirectories.Delete ( 0 );
          for li_i := 0 to lstl_Files.count-1 do
           if lstl_Files  [ li_i ] = FCurrentFile Then
            Begin
              li_j := li_i;
              while li_j < lstl_Files.count do
               Begin
                 if (( ImagesPerLine = 0 ) and ( PanelLine.Left - Self.HorzScrollBar.Position > Self.Width ))
                 or (( ImagesPerLine > 0 ) and ( PanelLine.Top  - Self.VertScrollBar.Position > Self.Height + PanelHeight ) and (CounterPerLine=ImagesPerLine))
                  Then
                   Begin
                     FDirectories.Add(LCurrentDir);
                     FCurrentFile := lstl_Files  [ li_j ];
                     lb_break := True;
                     Break;
                   end;
                  if FSubDirs and DirectoryExistsUTF8 ( lstl_Files [ li_j ] )
                   Then AddDirectory ( lstl_Files [ li_j ], FSubDirs )
                   Else AddFile(lstl_Files [ li_j ]);
                  inc ( li_j );
               end;
             Break;
           End;

          Inherited; // We create only now
        end;
     end;
  finally
    lstl_Files.Destroy;
  end;
end;

function TExtImages.CreateImagesMap: TExtCustomCollectionImages;
begin
  Result := TExtCollectionImages.Create(Self,TExtPanelImage);
  // adding editing component
  if csDesigning in ComponentState Then
    Result.Add;
end;

{$IFDEF VERSIONS}
initialization
  p_ConcatVersion ( gVer_TExtImages );
{$ENDIF}
end.
